'use strict';

/**
 * @file Main CLI object.
 * @author Fishine.Yuan
 */

/* eslint no-console:off */

var chalk = require('chalk');
var CliEngine = require('./cli-engine');
var initializer = require('./initializer');
var stylish = require('./formatters/stylish');
var options = require('./cli-options');

var ERROR_FIX_HELP = '\nUse \'git commit --no-verify\' to skip check.\nThese links are helpful to fix them:\n   eslint: ' + chalk.underline('http://eslint.org/docs/rules/') + '\n   eslint-plugin-react: ' + chalk.underline('https://github.com/yannickcr/eslint-plugin-react#list-of-supported-rules') + '\n   eslint-plugin-import: ' + chalk.underline('https://github.com/benmosher/eslint-plugin-import#rules') + '\n   eslint-plugin-jsx-a11y: ' + chalk.underline('https://github.com/evcohen/eslint-plugin-jsx-a11y#supported-rules') + '\n';

/**
 * Outputs the results of the linting
 * @param {Array.<Object>} results
 */
function printResults(results) {
    var stdout = stylish(results);

    if (stdout) {
        stdout += ERROR_FIX_HELP;
    }
    console.log(stdout ? stdout : 'Cilint pass');
}

/**
 * Encapsulates all CLI behavior for cilint. Makes it easier to test as well as
 * for other Node.js programs to effectively run the CLI.
 */
var cli = {
    /**
     * Executes the CLI based on an array of arguments that is passed in.
     * @param {String|Array|Object} args The arguments to process.
     * @return {Number} The exit code for the operation.
     */
    execute: function execute(args) {
        var currentOptions = void 0;
        try {
            currentOptions = options.parse(args);
        } catch (error) {
            console.log(error);
            return 1;
        }

        currentOptions.files = currentOptions._ || [];

        // set to current working dir
        if (!currentOptions.files.length) {
            currentOptions.files = ['.'];
        }

        if (currentOptions.help) {
            console.log(options.generateHelp());
            return 0;
        }

        if (currentOptions.init) {
            return this.init(currentOptions);
        }

        return this.lint(currentOptions);
    },


    /**
     * Run init
     * @param {Object} options The argument options to process
     * @return {Number} The exit code for the operation.
     */
    init: function init(options) {
        console.log(chalk.yellow('Cilint init..'));
        var results = initializer();
        results.copyed.forEach(function (item) {
            console.log('Generated', chalk.yellow(item));
        });
        console.log(chalk.yellow('Done'));

        return 0;
    },


    /**
     * Run lint
     * @param {Object} options The argument options to process
     * @return {Number} The exit code for the operation.
     */
    lint: function lint(options) {
        console.log('Cilint lint code style..');

        var engine = new CliEngine(options);

        var results = engine.getResults();

        printResults(results);

        var errors = 0;

        results.forEach(function (result) {
            result.messages.forEach(function (message) {
                if (message.isError) {
                    errors++;
                }
            });
        });

        return engine.rc.stopCommit !== false && errors ? 1 : 0;
    }
};

module.exports = cli;