'use strict';

var fs = require('fs-extra');
var path = require('path');
var request = require('request');
var requireFromString = require('require-from-string');
var deasync = require('deasync');

var PRECOMMIT_VERSION = '0.0.1';

/**
 * Wget exports from url
 * @param {String} url
 * @return {Object}
 */
function wget(url) {
    return deasync(function (done) {
        request({
            url: url,
            timeout: 3000
        }, function (error, response, body) {
            if (!error && response.statusCode === 200) {
                done(null, body);
            } else {
                // TODO log error
                done(null, null);
            }
        });
    })();
}

/**
 * Write rc file
 * @param {String} to
 * @param {...Object} args
 */
function writeRc(to) {
    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
    }

    var rc = Object.assign.apply(Object, [{}].concat(args));

    fs.writeFileSync(to, 'module.exports=' + JSON.stringify(rc, null, 4));
}

/**
 * Init cilintrc
 * @param {Object} options
 * @return {?String}
 */
function initCilintrc(options) {
    var rcPath = '.cilintrc.js';

    if (options.override || !fs.existsSync(rcPath)) {
        var url = options.cilintrcUrl;

        if (url) {
            var text = wget(url);
            if (text) {
                writeRc(rcPath, requireFromString(text), options.cilintrc);
                return rcPath;
            }
        } else {
            writeRc(rcPath, require('../conf/cilintrc.js'), options.cilintrc);
            return rcPath;
        }
    }
}

/**
 * Init eslintrc
 * @param {Object} options
 * @return {?String}
 */
function initEslintrc(options) {
    var rcPath = '.eslintrc.js';

    if (options.override || !fs.existsSync(rcPath)) {
        var url = options.eslintrcUrl;

        if (url) {
            var text = wget(url);
            if (text) {
                writeRc(rcPath, requireFromString(text), options.eslintrc);
                return rcPath;
            }
        } else {
            writeRc(rcPath, require('../conf/eslintrc.js'), options.eslintrc);
            return rcPath;
        }
    }
}

/**
 * Init editorconfig
 * @param {Object} options
 * @return {?String}
 */
function initEditorconfig(options) {
    var rcPath = '.editorconfig';

    if (options.override || !fs.existsSync(rcPath)) {
        var url = options.editorconfigUrl;

        if (url) {
            var text = wget(url);
            if (text) {
                fs.writeFileSync(rcPath, text);
                return rcPath;
            }
        }
    }
}

/**
 * Init precommit hook
 * @param {Object} options
 * @return {?String}
 */
function initPreCommit(options) {
    var from = path.join(__dirname, '../conf/pre-commit');
    var to = '.git/hooks/pre-commit';

    var version = void 0;
    if (fs.existsSync(to)) {
        version = fs.readFileSync(to).toString().match(/# CILint PreCommit v([\d\.]+)/) && RegExp.$1;
    }

    // check pre-commit version
    if (options.override || version !== PRECOMMIT_VERSION) {
        fs.copySync(from, to);
        return to;
    }
}

/**
 * Initializer
 * @param {Object} options
 * @param {?Boolean} options.override
 * @param {?Object} options.cilintrc
 * @param {?cilintrcUrl} options.cilintrcUrl
 * @param {?Object} options.eslintrc
 * @param {?eslintrcUrl} options.eslintrcUrl
 */
var initializer = function initializer() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var initers = [initCilintrc, initEslintrc, initEditorconfig, initPreCommit];

    var copyed = [];

    initers.forEach(function (fn) {
        var file = fn(options);
        if (file) {
            copyed.push(file);
        }
    });

    return {
        copyed: copyed
    };
};

module.exports = initializer;